@@ -13,6 +13,7 @@ from matplotlib import pyplot as pl, rcParams, ticker
 from matplotlib import patheffects
 from matplotlib.axes import Axes
 from matplotlib.colors import is_color_like, Colormap
+from scipy.sparse import issparse
 
 from .. import _utils
 from .._utils import matrix, _IGraphLayout, _FontWeight, _FontSize
@@ -352,9 +353,8 @@ def paga(
         graph is connected). If this is `None` or an empty list, the root
         vertices are automatically calculated based on topological sorting.
     transitions
-        Key for `.uns['paga']` that specifies the matrix that – for instance
-        `'transistions_confidence'` – that specifies the matrix that stores the
-        arrows.
+        Key for `.uns['paga']` that specifies the matrix that stores the
+        arrows, for instance `'transitions_confidence'`.
     solid_edges
         Key for `.uns['paga']` that specifies the matrix that stores the edges
         to be drawn solid black.
@@ -416,6 +416,24 @@ def paga(
     If `show==False`, one or more :class:`~matplotlib.axes.Axes` objects.
     Adds `'pos'` to `adata.uns['paga']` if `add_pos` is `True`.
 
+    Examples
+    --------
+
+    .. plot::
+        :context: close-figs
+
+        import scanpy as sc
+        adata = sc.datasets.pbmc3k_processed()
+        sc.tl.paga(adata, groups='louvain')
+        sc.pl.paga(adata)
+
+    You can increase node and edge sizes by specifying additional arguments.
+
+    .. plot::
+        :context: close-figs
+
+        sc.pl.paga(adata, node_size_scale=10, edge_width_scale=2)
+
     Notes
     -----
     When initializing the positions, note that – for some reason – igraph
@@ -1097,12 +1115,12 @@ def paga_path(
                 ]
             )
             idcs = idcs[idcs_group]
-            if key in adata.obs_keys():
-                x += list(adata.obs[key].values[idcs])
-            else:
-                x += list(adata_X[:, key].X[idcs])
+            values = (
+                adata.obs[key].values if key in adata.obs_keys() else adata_X[:, key].X
+            )[idcs]
+            x += (values.A if issparse(values) else values).tolist()
             if ikey == 0:
-                groups += [group for i in range(len(idcs))]
+                groups += [group] * len(idcs)
                 x_tick_locs.append(len(x))
                 for anno in annotations:
                     series = adata.obs[anno]
@@ -1128,7 +1146,7 @@ def paga_path(
                 else:
                     label = group
                 x_tick_labels.append(label)
-    X = np.array(X)
+    X = np.asarray(X).squeeze()
     if as_heatmap:
         img = ax.imshow(X, aspect='auto', interpolation='nearest', cmap=color_map)
         if show_yticks:
