@@ -1,6 +1,6 @@
 import collections.abc as cabc
 from copy import copy
-from typing import Union, Optional, Sequence, Any, Mapping, List, Tuple, Callable
+from typing import Union, Optional, Sequence, Any, Mapping, List, Tuple
 
 import numpy as np
 import pandas as pd
@@ -13,7 +13,7 @@ from matplotlib import pyplot as pl, colors
 from matplotlib.cm import get_cmap
 from matplotlib import rcParams
 from matplotlib import patheffects
-from matplotlib.colors import Colormap
+from matplotlib.colors import Colormap, Normalize
 from functools import partial
 
 from .. import _utils
@@ -21,9 +21,11 @@ from .._utils import (
     _IGraphLayout,
     _FontWeight,
     _FontSize,
-    circles,
     ColorLike,
+    VBound,
+    circles,
     check_projection,
+    check_colornorm,
 )
 from .._docs import (
     doc_adata_color_etc,
@@ -37,8 +39,6 @@ from ..._settings import settings
 from ..._utils import sanitize_anndata, _doc_params, Empty, _empty
 from ..._compat import Literal
 
-VMinMax = Union[str, float, Callable[[Sequence[float]], float]]
-
 
 @_doc_params(
     adata_color_etc=doc_adata_color_etc,
@@ -76,8 +76,10 @@ def embedding(
     legend_fontweight: Union[int, _FontWeight] = 'bold',
     legend_loc: str = 'right margin',
     legend_fontoutline: Optional[int] = None,
-    vmax: Union[VMinMax, Sequence[VMinMax], None] = None,
-    vmin: Union[VMinMax, Sequence[VMinMax], None] = None,
+    vmax: Union[VBound, Sequence[VBound], None] = None,
+    vmin: Union[VBound, Sequence[VBound], None] = None,
+    vcenter: Union[VBound, Sequence[VBound], None] = None,
+    norm: Union[Normalize, Sequence[Normalize], None] = None,
     add_outline: Optional[bool] = False,
     outline_width: Tuple[float, float] = (0.3, 0.05),
     outline_color: Tuple[str, str] = ('black', 'white'),
@@ -202,6 +204,10 @@ def embedding(
         vmax = [vmax]
     if isinstance(vmin, str) or not isinstance(vmin, cabc.Sequence):
         vmin = [vmin]
+    if isinstance(vcenter, str) or not isinstance(vcenter, cabc.Sequence):
+        vcenter = [vcenter]
+    if isinstance(norm, Normalize) or not isinstance(norm, cabc.Sequence):
+        norm = [norm]
 
     if 's' in kwargs:
         size = kwargs.pop('s')
@@ -288,13 +294,18 @@ def embedding(
                 )
                 ax.set_title(value_to_plot)
 
-        # check vmin and vmax options
-        if categorical:
-            kwargs['vmin'] = kwargs['vmax'] = None
-        else:
-            kwargs['vmin'], kwargs['vmax'] = _get_vmin_vmax(
-                vmin, vmax, count, color_vector
+        if not categorical:
+            vmin_float, vmax_float, vcenter_float, norm_obj = _get_vboundnorm(
+                vmin, vmax, vcenter, norm, count, color_vector
             )
+            normalize = check_colornorm(
+                vmin_float,
+                vmax_float,
+                vcenter_float,
+                norm_obj,
+            )
+        else:
+            normalize = None
 
         # make the scatter plot
         if projection == '3d':
@@ -305,6 +316,7 @@ def embedding(
                 marker=".",
                 c=color_vector,
                 rasterized=settings._vector_friendly,
+                norm=normalize,
                 **kwargs,
             )
         else:
@@ -347,6 +359,7 @@ def embedding(
                     marker=".",
                     c=bg_color,
                     rasterized=settings._vector_friendly,
+                    norm=normalize,
                     **kwargs,
                 )
                 ax.scatter(
@@ -356,6 +369,7 @@ def embedding(
                     marker=".",
                     c=gap_color,
                     rasterized=settings._vector_friendly,
+                    norm=normalize,
                     **kwargs,
                 )
                 # if user did not set alpha, set alpha to 0.7
@@ -367,6 +381,7 @@ def embedding(
                 marker=".",
                 c=color_vector,
                 rasterized=settings._vector_friendly,
+                norm=normalize,
                 **kwargs,
             )
 
@@ -463,14 +478,16 @@ def _panel_grid(hspace, wspace, ncols, num_panels):
     return fig, gs
 
 
-def _get_vmin_vmax(
-    vmin: Sequence[VMinMax],
-    vmax: Sequence[VMinMax],
+def _get_vboundnorm(
+    vmin: Sequence[VBound],
+    vmax: Sequence[VBound],
+    vcenter: Sequence[VBound],
+    norm: Sequence[Normalize],
     index: int,
     color_vector: Sequence[float],
 ) -> Tuple[Union[float, None], Union[float, None]]:
     """
-    Evaluates the value of vmin and vmax, which could be a
+    Evaluates the value of vmin, vmax and vcenter, which could be a
     str in which case is interpreted as a percentile and should
     be specified in the form 'pN' where N is the percentile.
     Eg. for a percentile of 85 the format would be 'p85'.
@@ -492,11 +509,12 @@ def _get_vmin_vmax(
     Returns
     -------
 
-    (vmin, vmax) containing None or float values
+    (vmin, vmax, vcenter, norm) containing None or float values for
+    vmin, vmax, vcenter and matplotlib.colors.Normalize  or None for norm.
 
     """
     out = []
-    for v_name, v in [('vmin', vmin), ('vmax', vmax)]:
+    for v_name, v in [('vmin', vmin), ('vmax', vmax), ('vcenter', vcenter)]:
         if len(v) == 1:
             # this case usually happens when the user sets eg vmax=0.9, which
             # is internally converted into list of len=1, but is expected that this
@@ -544,6 +562,7 @@ def _get_vmin_vmax(
                     )
                     v_value = None
         out.append(v_value)
+    out.append(norm[0] if len(norm) == 1 else norm[index])
     return tuple(out)
 
 
@@ -599,6 +618,43 @@ def umap(adata, **kwargs) -> Union[Axes, List[Axes], None]:
     Returns
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
+
+    Examples
+    --------
+
+    .. plot::
+        :context: close-figs
+
+        import scanpy as sc
+        adata = sc.datasets.pbmc68k_reduced()
+        sc.pl.umap(adata)
+
+    Colour points by discrete variable (Louvain clusters).
+
+    .. plot::
+        :context: close-figs
+
+        sc.pl.umap(adata, color="louvain")
+
+    Colour points by gene expression.
+
+    .. plot::
+        :context: close-figs
+
+        sc.pl.umap(adata, color="HES4")
+
+    Plot muliple umaps for different gene expressions.
+
+    .. plot::
+        :context: close-figs
+
+        sc.pl.umap(adata, color=["HES4", "TNFRSF4"])
+
+    .. currentmodule:: scanpy
+
+    See also
+    --------
+    tl.umap
     """
     return embedding(adata, 'umap', **kwargs)
 
@@ -624,6 +680,22 @@ def tsne(adata, **kwargs) -> Union[Axes, List[Axes], None]:
     Returns
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
+
+    Examples
+    --------
+    .. plot::
+        :context: close-figs
+
+        import scanpy as sc
+        adata = sc.datasets.pbmc68k_reduced()
+        sc.tl.tsne(adata)
+        sc.pl.tsne(adata, color='bulk_labels')
+
+    .. currentmodule:: scanpy
+
+    See also
+    --------
+    tl.tsne
     """
     return embedding(adata, 'tsne', **kwargs)
 
@@ -647,6 +719,22 @@ def diffmap(adata, **kwargs) -> Union[Axes, List[Axes], None]:
     Returns
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
+
+    Examples
+    --------
+    .. plot::
+        :context: close-figs
+
+        import scanpy as sc
+        adata = sc.datasets.pbmc68k_reduced()
+        sc.tl.diffmap(adata)
+        sc.pl.diffmap(adata, color='bulk_labels')
+
+    .. currentmodule:: scanpy
+
+    See also
+    --------
+    tl.diffmap
     """
     return embedding(adata, 'diffmap', **kwargs)
 
@@ -677,6 +765,22 @@ def draw_graph(
     Returns
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
+
+    Examples
+    --------
+    .. plot::
+        :context: close-figs
+
+        import scanpy as sc
+        adata = sc.datasets.pbmc68k_reduced()
+        sc.tl.draw_graph(adata)
+        sc.pl.draw_graph(adata, color=['phase', 'bulk_labels'])
+
+    .. currentmodule:: scanpy
+
+    See also
+    --------
+    tl.draw_graph
     """
     if layout is None:
         layout = str(adata.uns['draw_graph']['params']['layout'])
@@ -721,6 +825,37 @@ def pca(
     Returns
     -------
     If `show==False` a :class:`~matplotlib.axes.Axes` or a list of it.
+
+    Examples
+    --------
+
+    .. plot::
+        :context: close-figs
+
+        import scanpy as sc
+        adata = sc.datasets.pbmc3k_processed()
+        sc.pl.pca(adata)
+
+    Colour points by discrete variable (Louvain clusters).
+
+    .. plot::
+        :context: close-figs
+
+        sc.pl.pca(adata, color="louvain")
+
+    Colour points by gene expression.
+
+    .. plot::
+        :context: close-figs
+
+        sc.pl.pca(adata, color="CST3")
+
+    .. currentmodule:: scanpy
+
+    See also
+    --------
+    tl.pca
+    pp.pca
     """
     if not annotate_var_explained:
         return embedding(
@@ -1127,8 +1262,13 @@ def _color_vector(
     if not is_categorical_dtype(values):
         return values, False
     else:  # is_categorical_dtype(values)
-        color_map = _get_palette(adata, values_key, palette=palette)
-        color_vector = values.map(color_map).map(to_hex)
+        color_map = {
+            k: to_hex(v)
+            for k, v in _get_palette(adata, values_key, palette=palette).items()
+        }
+        # If color_map does not have unique values, this can be slow as the
+        # result is not categorical
+        color_vector = values.map(color_map)
 
         # Set color to 'missing color' for all missing values
         if color_vector.isna().any():
