@@ -13,6 +13,7 @@ from .. import logging as logg
 from ..preprocessing._simple import _get_mean_var
 from .._compat import Literal
 from ..get import _get_obs_rep
+from .._utils import check_nonnegative_integers
 
 
 _Method = Optional[Literal['logreg', 't-test', 'wilcoxon', 't-test_overestim_var']]
@@ -428,7 +429,7 @@ class _RankGenes:
 def rank_genes_groups(
     adata: AnnData,
     groupby: str,
-    use_raw: bool = True,
+    use_raw: Optional[bool] = None,
     groups: Union[Literal['all'], Iterable[str]] = 'all',
     reference: str = 'rest',
     n_genes: Optional[int] = None,
@@ -532,6 +533,11 @@ def rank_genes_groups(
     >>> # to visualize the results
     >>> sc.pl.rank_genes_groups(adata)
     """
+    if use_raw is None:
+        use_raw = adata.raw is not None
+    elif use_raw is True and adata.raw is not None:
+        raise ValueError("Received `use_raw=True`, but `adata.raw` is empty.")
+
     if method is None:
         logg.warning(
             "Default of the method has been changed to 't-test' from 't-test_overestim_var'"
@@ -583,6 +589,12 @@ def rank_genes_groups(
 
     test_obj = _RankGenes(adata, groups_order, groupby, reference, use_raw, layer, pts)
 
+    if check_nonnegative_integers(test_obj.X) and method != 'logreg':
+        logg.warning(
+            "It seems you use rank_genes_groups on the raw count data. "
+            "Please logarithmize your data before calling rank_genes_groups."
+        )
+
     # for clarity, rename variable
     n_genes_user = n_genes
     # make sure indices are not OoB in case there are less genes than n_genes
